VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsEXE"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'This is the class that reads info from an EXE.
'I made it as a class module so it's more reusable.

'I also left out error handling in the sub routines intentionally.
'All error handling/checking should be done before calling on subs in this class (or any class).

'Container for start of info property.
'This tells us where the information in the EXE starts.
Private strStart As String

'Get a chunk from a file and return it as a byte array.
Private Function GetFileChunk(ByVal FilePath As String, ByVal Offset As Long, ByVal Length As Long) As Byte()
    Dim lonFF As Long, bytData() As Byte
    
    lonFF = FreeFile
    ReDim bytData(1 To Length) As Byte
    
    Open FilePath For Binary Access Read As #lonFF
        Get #lonFF, Offset, bytData()
    Close #lonFF
    
    GetFileChunk = bytData()
    'Erase bytData()
End Function

'Check to see if a byte array is equal to the start info.
Private Function IsStartInfo(ByteData() As Byte) As Boolean
    Dim strTest As String
    
    'Convert byte array to string for easy comparison.
    strTest = StrConv(ByteData(), vbUnicode)
    
    If strTest = StartInfo Then
        IsStartInfo = True
    End If
    
End Function

'Find the start of the info in the EXE file.
'Returns the position where the actual info starts (not where the StartInfo variable starts).
Private Function FindInfoStart(ByVal EXEPath As String) As Long
    Dim lonFF As Long, lonLen As Long
    
    'Get an available file handle.
    lonFF = FreeFile
    'Get the size of the file in bytes.
    lonLen = FileLen(EXEPath)
    'Size of data to read at once in bytes.
    Dim lonPackSize As Long
    'Loop variable.
    Dim lonLoop As Long
    'Chunk of file to check for start info.
    Dim bytCheck() As Byte
    
    'Since we're writing the data to the END of the EXE file, we'll search from
    'the end to the beginning. This way, we don't waste time searching through data that we don't need to.
    'Also, it will always retrieve the latest info in the EXE file.
    
    'We'll set a packet size equal to the length of the info start (strStart).
    'We're only searching for that right now.
    
    lonPackSize = Len(StartInfo)
    
    'Allocate some memory to store the chunk of the file in.
    ReDim bytCheck(1 To lonPackSize) As Byte
    
    'Start looping through the file backwards.
    For lonLoop = ((lonLen - lonPackSize) - 1) To 1 Step -1
        bytCheck() = GetFileChunk(EXEPath, lonLoop, lonPackSize)
        
        'Check if we found the start info.
        If IsStartInfo(bytCheck()) = True Then
            'We found it.
            FindInfoStart = lonLoop + lonPackSize
            Exit For
        End If
    
    Next lonLoop
    
End Function

'Read the info from the EXE file and return it as a byte array.
Public Function ReadInfo(ByVal EXEPath As String) As Byte()
    Dim lonInfoStart As Long
    Dim lonSize As Long, lonFF As Long
    
    'Find where the info starts.
    lonInfoStart = FindInfoStart(EXEPath)
    'Get the size of the EXE file in bytes.
    lonSize = FileLen(EXEPath)
    
    If lonInfoStart > 0 Then
        'Found the info.
        'Allocate memory to store the info.
        ReadInfo = GetFileChunk(EXEPath, lonInfoStart, (lonSize - lonInfoStart) + 1)
    End If
    
End Function

'Write info to an EXE file.
'I left WriteData as a variant so you can pass any kind of data, array, byte, string, etc.
'If you already know the kind of data you'll be writing, then replace variant with that type.
'I also left out error handling so you can see if this class is raising errors.
'Proper error handling should be done in your app before calling this sub.
'ie: Check if the file exists, etc.
Public Sub WriteInfo(ByVal EXEPath As String, ByVal WriteData As Variant)
    Dim lonFF As Long, lonLen As Long
    
    'Get an available file handle.
    lonFF = FreeFile
    'Get the size of the file in bytes.
    lonLen = FileLen(EXEPath)
    
    'Open the EXE file in binary write mode because it is a binary file.
    Open EXEPath For Binary Access Write As #lonFF
        'Write the data to the end of the file.
        Put #lonFF, (lonLen + 1), WriteData
    Close #lonFF 'Close the file.
    
End Sub

'I could have just used public variables instead of properties,
'but it's better coding practice to use properties in classes.
Public Property Get StartInfo() As String
    Let StartInfo = strStart
End Property

Public Property Let StartInfo(ByVal NewValue As String)
    Let strStart = NewValue
End Property
